"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const argnames_1 = require("@chengaoyuan/argnames");
class Cache {
    constructor(protocol) {
        this.protocol = protocol;
    }
    Cacheable(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    var $key$ = ${config.key ? config.key : ""};
                    return new Promise(function(resolve, reject){
                        var $protocol$ = $info$.protocol;
                        $protocol$.get($cache$, $key$, function($cacheValue$) {
                            if ($cacheValue$ !== undefined) {
                                resolve($cacheValue$);
                            } else {
                                $function$.call($self$${args ? ", " + args : ""})
                                        .then(function($retValue$){
                                            if ($retValue$ !== undefined) {
                                                if(${config.condition ? config.condition : true}) {
                                                    $protocol$.set($cache$, $key$, ${config.expire}, $retValue$);
                                                }
                                            }
                                            resolve($retValue$);
                                        }, reject);
                            }
                        });
                    })
                }
            `)(des.value, this, config.cache);
        };
    }
    CachePut(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    var $key$ = ${config.key ? config.key : ""};
                    return new Promise(function(resolve, reject){
                        var $protocol$ = $info$.protocol;
                        $function$.call($self$${args ? ", " + args : ""})
                                .then(function($retValue$){
                                    if ($retValue$ !== undefined) {
                                        if(${config.condition ? config.condition : true}) {
                                            $protocol$.set($cache$, $key$, ${config.expire}, $retValue$);
                                        }
                                    }
                                    resolve($retValue$);
                                }, reject);
                    })
                }
            `)(des.value, this, config.cache);
        };
    }
    CacheEvict(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    var $key$ = ${config.key ? config.key : ""};
                    return new Promise(function(resolve, reject){
                        var $protocol$ = $info$.protocol;
                        $function$.call($self$${args ? ", " + args : ""})
                                .then(function($retValue$){
                                    resolve($retValue$);
                                    if(${config.condition ? config.condition : true}) {
                                        $protocol$.del($cache$, $key$);
                                    }
                                }, reject);
                    })
                }
            `)(des.value, this, config.cache);
        };
    }
    CacheEvictMulti(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    return new Promise(function(resolve, reject){
                        var $protocol$ = $info$.protocol;
                        $function$.call($self$${args ? ", " + args : ""})
                                .then(function($retValue$){
                                    resolve($retValue$);
                                    for(var ${config.foreach}) {
                                        if(${config.condition ? config.condition : true}) {
                                            var $key$ = ${config.key ? config.key : ""};
                                            $protocol$.del($cache$, $key$);
                                        }
                                    }
                                }, reject);
                    })
                }
            `)(des.value, this, config.cache);
        };
    }
    CacheEvictAll(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    return new Promise(function(resolve, reject){
                        var $protocol$ = $info$.protocol;
                        $function$.call($self$${args ? ", " + args : ""})
                                .then(function($retValue$){
                                    resolve($retValue$);
                                    if(${config.condition ? config.condition : true}) {
                                        $protocol$.clear($cache$);
                                    }
                                }, reject);
                    })
                }
            `)(des.value, this, config.cache);
        };
    }
    CacheableLock(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            const lockTime = config.lockTime ? config.lockTime : 3;
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    var $key$ = ${config.key ? config.key : ""};
                    return new Promise(function(resolve, reject) {
                        var $protocol$ = $info$.protocol;
                        var $locker$ = $protocol$.create_locker($cache$, $key$);
                        $locker$.lock(${lockTime}, function($getLockResult$) {
                            if($getLockResult$) {
                                $protocol$.get($cache$, $key$, function($cacheValue$) {
                                    if ($cacheValue$ !== undefined) {
                                        $locker$.unlock();
                                        resolve($cacheValue$);
                                    } else {
                                        if(!(${config.condition ? config.condition : true})) {
                                            $locker$.unlock();
                                        }
                                        $function$.call($self$${args ? ", " + args : ""})
                                                .then(function($retValue$){
                                                    if ($retValue$ !== undefined) {
                                                        if(${config.condition ? config.condition : true}) {
                                                            $protocol$.set($cache$, $key$, ${config.expire}, $retValue$);
                                                        }
                                                    }
                                                    $locker$.unlock();
                                                    resolve($retValue$);
                                                }, function($callerr$){
                                                    $locker$.unlock();
                                                    reject($callerr$);
                                                });
                                    }
                                });
                            } else {
                                $function$.call($self$${args ? ", " + args : ""})
                                    .then(resolve, reject);
                            }
                            
                        });
                    });
                };
            `)(des.value, this, config.cache);
        };
    }
    CacheEvictLock(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            const lockTime = config.lockTime ? config.lockTime : 3;
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    var $key$ = ${config.key ? config.key : ""};
                    return new Promise(function(resolve, reject){
                        var $protocol$ = $info$.protocol;
                        $function$.call($self$${args ? ", " + args : ""})
                                .then(function($retValue$){
                                    resolve($retValue$);
                                    if(${config.condition ? config.condition : true}) {
                                        var $locker$ = $protocol$.create_locker($cache$, $key$);
                                        $locker$.lock(${lockTime}, function($delLockResult$) {
                                            if($delLockResult$) {
                                                $protocol$.del($cache$, $key$);
                                                $locker$.unlock();
                                            }
                                        });
                                    }
                                }, reject);
                    })
                }
            `)(des.value, this, config.cache);
        };
    }
    CacheEvictMultiLock(config) {
        return (target, methodName, des) => {
            const args = argnames_1.default(des.value).join(",");
            const lockTime = config.lockTime ? config.lockTime : 3;
            des.value = new Function("$function$", "$info$", "$cache$", `
                return function(${args}) {
                    var $self$ = this;
                    return new Promise(function(resolve, reject){
                        var $protocol$ = $info$.protocol;
                        $function$.call($self$${args ? ", " + args : ""})
                                .then(function($retValue$){
                                    resolve($retValue$);
                                    var $record$ = {};
                                    var $delfunc$ = function($key$) {
                                        if($record$[$key$]) {
                                            return;
                                        }
                                        $record$[$key$] = true;
                                        var $locker$ = $protocol$.create_locker($cache$, $key$);
                                        $locker$.lock(${lockTime}, function($delLockResult$) {
                                            if($delLockResult$) {
                                                $protocol$.del($cache$, $key$);
                                                $locker$.unlock();
                                            }
                                        });
                                    };
                                    
                                    for(var ${config.foreach}) {
                                        if(${config.condition ? config.condition : true}) {
                                            $delfunc$(${config.key ? config.key : ""})
                                        }
                                    }
                                }, reject);
                    })
                }
            `)(des.value, this, config.cache);
        };
    }
}
exports.Cache = Cache;
//# sourceMappingURL=index.js.map